(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('vue')) :
	typeof define === 'function' && define.amd ? define(['vue'], factory) :
	(factory(global.Vue));
}(this, (function (Vue) { 'use strict';

Vue = Vue && Vue.hasOwnProperty('default') ? Vue['default'] : Vue;

/**
 * Helper function to determine a friendly string to represent the given Vue instance
 *
 * @param {Object} vueInstance The instance in question
 * @returns {String} Components display name
 */
function getStringName (vueInstance) {
    let display = 'Anonymous Component';

    const tagName = vueInstance.$options._componentTag,
        el = vueInstance.$options.el;

    if (tagName) {
        display = `<${tagName}>`;
    } else if (el) {
        display = `el: ${el}`;
    }

    return display;
}

/**
 * Logs a message to the console with additional info about the Vue instance
 *
 * @param {Object} vueInstance Reference to the calling instance
 * @param {String} msg Message to be logged
 * @param {String} type Type of message to log (defaults to 'warn'), could be 'warn', 'error' or 'log'
 */
function log (vueInstance, msg, type = 'warn') {
    const componentName = getStringName(vueInstance);

    // eslint-disable-next-line no-console
    console[type](`${componentName} - ${msg}`);
}

/**
 * Check that given value is a type of list (Array, Map or Set).
 *
 * @param  {Boolean | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function list (ruleValue, dataValue) {
    let isValid = true;
    const errorMessage = ruleValue.message || 'Does not meet the `list` rule.';
        // value = ruleValue.value || ruleValue;

    isValid = (Array.isArray(dataValue)) ||
            (dataValue instanceof Set) || (dataValue instanceof WeakSet) ||
            (dataValue instanceof  Map) || (dataValue instanceof  WeakMap);

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Checks that the given value actually has a value.
 * In the case of lists, makes sure they are not empty
 *
 * @param  {Boolean | Object} ruleValue Rule configuration
 * @param {String | Number | Array | Boolean | Object} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function required (ruleValue, dataValue) {
    const empty = 0,
        // value = ruleValue.value || ruleValue,
        isList = list(ruleValue, dataValue).isValid,
        errorMessage = ruleValue.message || 'Does not meet the `required` rule.';

    let isValid = true;

    if (dataValue === null || dataValue === undefined) {
        isValid = false;
    } else if (typeof dataValue === 'string' || isList) {
        isValid = dataValue.length > empty;
    } else if (dataValue instanceof Object) {
        isValid = Object.keys(dataValue).length > empty  && Object.values(dataValue)
                // filter out falsy values (like empty strings, null, etc)
                // eslint-disable-next-line arrow-parens
                .filter(val => val)
                .length > empty;
    }

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Check if the list or string meets the minimum length.
 * Or if a number meets the minimum value.
 *
 * @param  {Number | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function min (ruleValue, dataValue) {
    let isValid = true;
    const value = ruleValue.value || ruleValue,
        isList = list(ruleValue, dataValue).isValid,
        errorMessage = ruleValue.message || 'Does not meet the `min` rule.';

    if (typeof dataValue === 'string' || isList) {
        isValid = dataValue.length >= value;
    } else if (typeof dataValue === 'number') {
        isValid =  dataValue >= value;
    } else {
        isValid = false;
    }


    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Check if the list, string or number does not surpass the maximum length;
 *
 * @param  {Number | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function max (ruleValue, dataValue) {
    let isValid = true;
    const value = ruleValue.value || ruleValue,
        isList = list(ruleValue, dataValue).isValid,
        errorMessage = ruleValue.message || 'Does not meet the `max` rule.';

    if (typeof dataValue === 'string' || isList) {
        isValid = dataValue.length <= value;
    } else if (typeof dataValue === 'number') {
        isValid =  dataValue <= value;
    } else {
        isValid = false;
    }


    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Checks that the given value is a number.
 *
 * @param  {Number | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function numeric (ruleValue, dataValue) {
    const errorMessage = ruleValue.message || 'Does not meet the `numeric` rule.',
        // value = ruleValue.value || ruleValue,
        isValid = typeof dataValue === 'number';

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Check that the value contains letter characters only, no numbers or special characters
 *
 * @param  {Boolean | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function alphabetic (ruleValue, dataValue) {
    const expression = RegExp('^[A-Za-z ]+$'),
        // value = ruleValue.value || ruleValue,
        isValid = expression.test(dataValue),
        errorMessage = ruleValue.message || 'Does not meet the `alphabetic` rule.';

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Check that the value contains numbers or letters, but no special characters.
 *
 * @param  {Boolean | Object} ruleValue Rule configuration
 * @param {*} dataValue Attributes current value
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function alphanumeric (ruleValue, dataValue) {

    const expression = RegExp('^[A-Za-z0-9 ]+$'),
        // value = ruleValue.value || ruleValue,
        isValid = expression.test(dataValue),
        errorMessage = ruleValue.message || 'Does not meet the `alphanumeric` rule.';

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Check if the two given arrays are the same by doing a comparison of their individuals items.
 * It takes in to account the order of its content.
 *
 * @param {Array} dataValue The base array
 * @param {Array} partnerValue The array to be compared
 * @returns {boolean} True if they are the same, otherwise False
 */
function compareArray (dataValue, partnerValue) {
    let isValid = false;

    if (dataValue && partnerValue) {
        // find miss-matched values.
        // the "gap" here is that if one list is longer than the other... then the miss-match might not be detected.
        // but there is a check for that later on.
        const missMatchedValues = dataValue.filter((val, index) => {
            let missMatchVal = undefined;

            if (Array.isArray(val) || Array.isArray(partnerValue[index])) {
                // if values at the given index are arrays... go in and check those too
                // because the "partner value" could be shorter than the value its self... we set a fallback value of
                // empty array.
                missMatchVal = !compareArray(val, partnerValue[index] || []);
            } else if (val instanceof Object || partnerValue[index] instanceof Object) {
                missMatchVal = !compareObject(val, partnerValue[index]);
            } else {
                missMatchVal = val !== partnerValue[index];
            }

            return missMatchVal;
        });

        // must be the same length
        if (dataValue.length !== partnerValue.length ||
            // must have the same content in the same order
            missMatchedValues.length) {

            isValid = false;
        } else {
            isValid = true;
        }
    }

    return isValid;
}

/**
 * Checks if the two given objects are the same by doing a comparison of each individual attribute
 *
 * @param {Object} dataValue The base value
 * @param {Object} partnerValue The value to be compared
 * @returns {boolean} True if they are the same, otherwise False
 */
function compareObject (dataValue, partnerValue) {

    let isValid = false;

    if (dataValue && partnerValue) {
        // generate an array of keys and an array of values
        // sorting so that order of keys will not influence the result
        const dataKeys = Object.keys(dataValue).sort(),
            dataValues = Object.values(dataValue).sort(),
            partnerKeys = Object.keys(partnerValue).sort(),
            partnerValues = Object.values(partnerValue).sort(),

            // now that we have an array of strings... just treat them as normal arrays :-)
            doKeysMatch = compareArray(dataKeys, partnerKeys),
            doValuesMatch = compareArray(dataValues, partnerValues);

        isValid = doKeysMatch && doValuesMatch;
    }

    return isValid;


}

/**
 * Checks that the given value matches the value of the companion field.
 * Applies to list s as well.
 *
 * @param  {String | Object} ruleValue Rule configuration
 * @param {String | Number | Array} dataValue Attributes current value
 * @param {Object} componentData All data attributes on the component
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function sameAs (ruleValue, dataValue, componentData) {
    const value     = ruleValue.value || ruleValue,
        isValueList = list({}, dataValue).isValid,

        partnerValue    = componentData[value],
        isPartnerList   = list({}, partnerValue).isValid,

        errorMessage = ruleValue.message || 'Does not meet the `sameAs` rule.';

    let isValid = false;

    if (isValueList && isPartnerList) {
        isValid = compareArray(dataValue, partnerValue);
    } else if (dataValue instanceof Object && partnerValue instanceof Object) {
        isValid = compareObject(dataValue, partnerValue);
    } else if (dataValue === partnerValue) {
        isValid = true;
    }

    return {
        isValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Checks that at least one of the fields is not empty.
 *
 * @param {String | Object} ruleValue Rule configuration
 * @param {String | Number | Array} fieldValue Attributes current value
 * @param {Object} componentData All data attributes on the component
 * @returns {{isValid: boolean, errorMessage: string}} Validation result object
 */
function or (ruleValue, fieldValue, componentData) {
    let isValid = false,
        isCompanionValid = false;

    const empty = 0,
        companionFieldName = ruleValue.value || ruleValue,
        companionValue = componentData[companionFieldName],
        isValueList = list({}, fieldValue).isValid,
        isPartnerList = list({}, companionValue).isValid,
        errorMessage = ruleValue.message || 'Does not meet the `or` rule.';


    // TODO: need to handle the case of Objects

    // first check if the primary field is non-empty
    if (isValueList || typeof fieldValue === 'string') {
        isValid = fieldValue.length > empty;
    } else if (fieldValue !== null && fieldValue !== undefined) {
        isValid = true;
    }

    // now do the same thing for the companion field
    if (isPartnerList || typeof companionValue === 'string') {
        isCompanionValid = companionValue.length > empty;
    } else if (companionValue !== null && companionValue !== undefined) {
        isCompanionValid = true;
    }

    return {
        // this rule is only valid if at least one of the fields are non-empty
        isValid: isValid || isCompanionValid,
        errorMessage: isValid ? '' : errorMessage
    };
}

/**
 * Classifies a components properties as "missing" or "toValidate" based on the list of validation rules that are to
 * be applied to the component.
 *
 * @param {Object} validationRules A "rules" object, as declared in the component
 * @param {Object} vmProperties Object representing the `data` and `computed` properties on the component
 * @returns {{missing: Array, toValidate: Array}} Object with 2 lists... indicating what needs to be validated vs
 * what is missing.
 */
function classifyProperties (validationRules, vmProperties) {
    const missing = [],
        toValidate = [];

    Object.keys(validationRules).forEach(ruleAttr => {
        if (vmProperties.hasOwnProperty(ruleAttr)) {
            toValidate.push(ruleAttr);
        } else {
            missing.push(ruleAttr);
        }
    });

    return {
        missing,
        toValidate
    };
}

/* eslint-disable no-invalid-this */

const initialValues = {};
const _validationObjectTemplate = {
        isDirty: false,
        isValid: true,
        isProcessing: false,
        rules: {},
        errors: new Set()
    };

let validators = {
    required,
    min,
    max,
    numeric,
    alphabetic,
    alphanumeric,
    list,
    sameAs,
    or
};

/**
 * Helper function to check if the given values have changed.
 * The only reason this is a helper and not just a simple `===` check... is so we can do a deep-compare for objects
 * and arrays. For this we leverage the existing logic defined in the `sameAs()` validator.
 *
 * @param {*} currentValue Present value
 * @param {*} initialValue Starting value
 * @returns {boolean} True if it has changed, otherwise false.
 */
function checkIfDirty (currentValue, initialValue) {
    const data = {
        currentValue,
        initialValue
    };

    // in this context, "isValid = true" means it has not changed while "isValid = false" means it has changed, so
    // we want to return the opposite of the validation.
    return !sameAs('initialValue', data.currentValue, data).isValid;
}

/**
 * Runs through all the necessary steps for configuring and setting up validations.
 * It is intended to be used with with a lifecycle hook.
 */
function setupValidations () {

    // setup local references for the component/instance properties we care about
    const vm = this,
        options = vm.$options,
        rules = options.validation.rules || {},
        componentData = Object.assign({}, options.data(), options.computed),
        validationResults = vm.$data.$validation;

    // remove references to its self
    delete componentData['$isValid'];
    delete componentData['$validation'];

    // combine custom validations with built-in validations
    validators = Object.assign({}, validators, options.validation.customValidations || {});

    // eslint-disable-next-line one-var
    const propertyClassification = classifyProperties(rules, componentData);

    if (propertyClassification.missing.length) {
        const logMsg = `Validations will not work for the follow attribute(s) because it was not found in the 'data'
             nor 'computed' properties: ${propertyClassification.missing.join(', ')}`;

        log(vm,  logMsg, 'warn');
    }

    // setup a watcher for each property that has a validation rule.
    propertyClassification.toValidate.forEach(attribute => {

        // keep a history of their initial value, used to initially determine the "dirtiness" of a property
        if (typeof componentData[attribute] === 'function') {
            // computed properties need to be treated as functions
            initialValues[attribute] = componentData[attribute]();
        } else {
            initialValues[attribute] = componentData[attribute];
        }

        // setup the initial validation result structure for each attribute
        // the JSON-stringify-parse thing is a hack to make a deep-copy
        vm.$set(validationResults, attribute, JSON.parse(JSON.stringify(_validationObjectTemplate)));

        const watcherOptions = {
            immediate: true
        };

        // when validating objects, setup a deep watching
        // ref: https://vuejs.org/v2/api/#vm-watch
        if (componentData[attribute] instanceof Object) {
            watcherOptions.deep = true;
        }

        // eslint-disable-next-line prefer-arrow-callback
        vm.$watch(attribute, async function validationTrigger (newValue) {
            const result = validationResults[attribute],
                initialValue = initialValues[attribute],
                applicableRules = rules[attribute],
                ruleNames = Object.keys(applicableRules);

            result.isValid = true;
            result.errors = new Set();


            // "dirty" means that the value has been modified by the user. even if the old and new value are the same...
            // ... it is still considered to be "dirty" because it has been tampered with. so once you become dirty...
            // ...there is no going back.
            result.isDirty = (result.isDirty) || checkIfDirty(newValue, initialValue);

            // this flag is to indicate that a particular attribute is in the process of being validated.
            // kind of pointless in this context since this is a synchronous validation, but absolutely need for
            // customer validation rules that may be async.
            result.isProcessing = true;

            // actually run all the rules.
            // we wait for all of them to be done so we can properly set `isProcessing`.
            // we also assume that they are sync... since we can not predict what users will do in custom validators
            // eslint-disable-next-line one-var
            const allResults = await Promise.all(ruleNames.map(async rule => {

                const ruleValue = applicableRules[rule],
                    vmProperties = Object.assign({}, vm.$data, vm.$options.computed),
                    res = await validators[rule](ruleValue, newValue, vmProperties);

                result.rules[rule] = res.isValid;

                if (!res.isValid) {
                    result.isValid = res.isValid;
                    result.errors.add(res.errorMessage);
                }

                return res;
            }));


            // if any of the results did not pass... mark everything as invalid
            // eslint-disable-next-line one-var
            const failureCount = allResults.filter(result => !result.isValid);
            result.isValid = !failureCount.length;

            result.errors = Array.from(result.errors);


            result.isProcessing = false;


        }, watcherOptions);

    });

}

var validation = {
    data () {
        return {
            // because of the `$`, this will only be accesable via `this.$data.$validation`. so to make things
            // convenient for the developer using the plugin... a computed property named "$validations" exposes this
            // data property.
            // ref: https://vuejs.org/v2/api/#data
            $validation: {}
        };

    },

    computed: {

        $validations () {
            return this.$data.$validation;
        },


        /**
         * Convenience property to check if any validations failed.
         *
         * @returns {Boolean} true if all validation passed, false if at least 1 failed.
         */
        $isValid () {
            // if anything with `isValid` = false... then the entire thing is invalid.
            const validationResults = Object.values(this.$validations).map(validation => validation.isValid);

            return !validationResults.includes(false);
        }
    },

    beforeCreate () {

        // no `validation` property? nothing to do here
        if (!this.$options.validation && !this.$options.validation.rules) {
            const logMsg = 'You are attempting to use the Validator plugin with no validations configured.';
            log(this, logMsg, 'error');
        }
    },

    created: setupValidations

};

var App = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('section',{staticClass:"section",attrs:{"id":"app"}},[_vm._m(0),_vm._v(" "),_c('form',{staticClass:"container"},[_c('label',[_vm._v(" Name: "),_c('input',{directives:[{name:"model",rawName:"v-model.trim",value:(_vm.name),expression:"name",modifiers:{"trim":true}}],attrs:{"type":"text"},domProps:{"value":(_vm.name)},on:{"input":function($event){if($event.target.composing){ return; }_vm.name=$event.target.value.trim();},"blur":function($event){_vm.$forceUpdate();}}})]),_vm._v(" "),_c('label',[_vm._v(" Age: "),_c('input',{directives:[{name:"model",rawName:"v-model.number",value:(_vm.age),expression:"age",modifiers:{"number":true}}],attrs:{"type":"number"},domProps:{"value":(_vm.age)},on:{"input":function($event){if($event.target.composing){ return; }_vm.age=_vm._n($event.target.value);},"blur":function($event){_vm.$forceUpdate();}}})]),_vm._v(" "),_c('label',[_vm._v(" Password: "),_c('input',{directives:[{name:"model",rawName:"v-model.number",value:(_vm.password),expression:"password",modifiers:{"number":true}}],attrs:{"type":"text"},domProps:{"value":(_vm.password)},on:{"input":function($event){if($event.target.composing){ return; }_vm.password=_vm._n($event.target.value);},"blur":function($event){_vm.$forceUpdate();}}}),_vm._v(" "),_c('input',{directives:[{name:"model",rawName:"v-model.number",value:(_vm.passwordConfirm),expression:"passwordConfirm",modifiers:{"number":true}}],attrs:{"type":"text"},domProps:{"value":(_vm.passwordConfirm)},on:{"input":function($event){if($event.target.composing){ return; }_vm.passwordConfirm=_vm._n($event.target.value);},"blur":function($event){_vm.$forceUpdate();}}})]),_vm._v(" "),_c('label',[_vm._v(" Favorite colors: "),_c('label',[_vm._v(" Red "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors),expression:"colors"}],attrs:{"type":"checkbox","value":"red"},domProps:{"checked":Array.isArray(_vm.colors)?_vm._i(_vm.colors,"red")>-1:(_vm.colors)},on:{"change":function($event){var $$a=_vm.colors,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="red",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Blue "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors),expression:"colors"}],attrs:{"type":"checkbox","value":"blue"},domProps:{"checked":Array.isArray(_vm.colors)?_vm._i(_vm.colors,"blue")>-1:(_vm.colors)},on:{"change":function($event){var $$a=_vm.colors,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="blue",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Green "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors),expression:"colors"}],attrs:{"type":"checkbox","value":"green"},domProps:{"checked":Array.isArray(_vm.colors)?_vm._i(_vm.colors,"green")>-1:(_vm.colors)},on:{"change":function($event){var $$a=_vm.colors,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="green",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Black "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors),expression:"colors"}],attrs:{"type":"checkbox","value":"black"},domProps:{"checked":Array.isArray(_vm.colors)?_vm._i(_vm.colors,"black")>-1:(_vm.colors)},on:{"change":function($event){var $$a=_vm.colors,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="black",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" White "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors),expression:"colors"}],attrs:{"type":"checkbox","value":"white"},domProps:{"checked":Array.isArray(_vm.colors)?_vm._i(_vm.colors,"white")>-1:(_vm.colors)},on:{"change":function($event){var $$a=_vm.colors,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="white",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors=$$c;}}}})])]),_vm._v(" "),_c('br'),_vm._v(" "),_c('label',[_vm._v(" Favorite colors: "),_c('label',[_vm._v(" Red "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors2),expression:"colors2"}],attrs:{"type":"checkbox","value":"red"},domProps:{"checked":Array.isArray(_vm.colors2)?_vm._i(_vm.colors2,"red")>-1:(_vm.colors2)},on:{"change":function($event){var $$a=_vm.colors2,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="red",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors2=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors2=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors2=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Blue "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors2),expression:"colors2"}],attrs:{"type":"checkbox","value":"blue"},domProps:{"checked":Array.isArray(_vm.colors2)?_vm._i(_vm.colors2,"blue")>-1:(_vm.colors2)},on:{"change":function($event){var $$a=_vm.colors2,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="blue",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors2=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors2=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors2=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Green "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors2),expression:"colors2"}],attrs:{"type":"checkbox","value":"green"},domProps:{"checked":Array.isArray(_vm.colors2)?_vm._i(_vm.colors2,"green")>-1:(_vm.colors2)},on:{"change":function($event){var $$a=_vm.colors2,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="green",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors2=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors2=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors2=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" Black "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors2),expression:"colors2"}],attrs:{"type":"checkbox","value":"black"},domProps:{"checked":Array.isArray(_vm.colors2)?_vm._i(_vm.colors2,"black")>-1:(_vm.colors2)},on:{"change":function($event){var $$a=_vm.colors2,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="black",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors2=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors2=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors2=$$c;}}}})]),_vm._v(" "),_c('label',[_vm._v(" White "),_c('input',{directives:[{name:"model",rawName:"v-model",value:(_vm.colors2),expression:"colors2"}],attrs:{"type":"checkbox","value":"white"},domProps:{"checked":Array.isArray(_vm.colors2)?_vm._i(_vm.colors2,"white")>-1:(_vm.colors2)},on:{"change":function($event){var $$a=_vm.colors2,$$el=$event.target,$$c=$$el.checked?(true):(false);if(Array.isArray($$a)){var $$v="white",$$i=_vm._i($$a,$$v);if($$el.checked){$$i<0&&(_vm.colors2=$$a.concat([$$v]));}else{$$i>-1&&(_vm.colors2=$$a.slice(0,$$i).concat($$a.slice($$i+1)));}}else{_vm.colors2=$$c;}}}})])]),_vm._v(" "),_c('div',{staticStyle:{"border":"1px solid red"}},[_c('pre',[_vm._v(_vm._s(_vm.$validations))])])])])},staticRenderFns: [function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('a',{attrs:{"href":"http://placecorgi.com/","target":"_blank"}},[_c('img',{attrs:{"src":"http://placecorgi.com/250"}})])}],
    mixins: [validation],
    validation: {
        rules: {
            mine: {}, yours: {},
            doubleAge: {
                numeric: true
            },
            password: {
                sameAs: 'passwordConfirm',
                required: true,
                myTest: true
            },
            name: {
                required: true,
                min: 10,
                alphabetic: true
            },
            age: {
                required: true,
                numeric: true,
                min: {
                    value: 18,
                    message: 'Too young...'
                },
                max: {
                    value: 100,
                    message: 'Old fart...'
                }
            },
            colors: {
                required: true,
                list: true,
                min: 1,
                max: 3,
                sameAs: 'colors2'
            }
        },
        customValidations: {
            myTest (ruleValue, dataValue, componentData) {
                return new Promise(function (resolve, reject) {

                    setTimeout(function () {
                        console.log('times up!');
                        resolve({isValid: false, errorMessage: 'my customer msg'});
                    }, 5000);
                });

            }
        }
    },

    data () {
        return {
            password: '',
            passwordConfirm: '',
            name: '',
            mine: '',
            yours: '',
            age: 8,
            colors: [],
            colors2: []
        };
    },

    watch: {
        async name () {
            await function () {
                
            };
        }
    },

    computed: {
        doubleAge () {
            return this.age * 2;
        }
    }
};

// The Vue build version to load with the `import` command
// (runtime-only or standalone) has been set in webpack.base.conf with an alias.
Vue.config.devtools = true;
/* eslint-disable no-new */
new Vue({
    el: '#app',
    // eslint-disable-next-line arrow-body-style
    render: h => h(App)
});

})));
